package org.bouncycastle.asn1.pkcs;

import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.x509.*;

/**
 * <pre>
 *      EnvelopedData ::= SEQUENCE {
 *           version Version,
 *			 recipientInfos RecipientInfos,
 *           encryptedContentInfo EncryptedContentInfo
 *      }
 *
 *		RecipientInfos ::= SET OF RecipientInfo
 *
 *      EncryptedContentInfo ::= SEQUENCE {
 *          contentType ContentType,
 *          contentEncryptionAlgorithm  ContentEncryptionAlgorithmIdentifier,
 *          encryptedContent [0] IMPLICIT EncryptedContent OPTIONAL
 *    }
 *
 *    EncryptedContent ::= OCTET STRING
 * </pre>
 */
public class EnvelopedData
    implements DEREncodable
{
	DERConstructedSequence		seq;
	DERConstructedSequence		data;
	DERConstructedSet			recipient;
    DERObjectIdentifier         bagId;
    DERObject                   bagValue;

	public EnvelopedData( DERConstructedSequence seq) {
		this.seq = seq;

		int version = ((DERInteger)seq.getObjectAt(0)).getValue().intValue();

        if (version != 0)
        {
            throw new IllegalArgumentException("sequence not version 0");
        }

		this.recipient  = (DERConstructedSet)seq.getObjectAt(1);
		this.data	    = (DERConstructedSequence)seq.getObjectAt(2);
	}

	public EnvelopedData(
		DERConstructedSet		recipientInfo,
		DERObjectIdentifier     contentType,
        AlgorithmIdentifier     encryptionAlgorithm,
        DERObject               content)
    {

		recipient = recipientInfo;
			
        data = new DERConstructedSequence();

        data.addObject(contentType);
        data.addObject(encryptionAlgorithm.getDERObject());

		if ( content != null )
        	data.addObject(new DERTaggedObject(false, 0, content));

    }

	public DERObjectIdentifier getContentType()
	{
		return (DERObjectIdentifier)data.getObjectAt(0);
	}

    public AlgorithmIdentifier getEncryptionAlgorithm()
    {
		return new AlgorithmIdentifier((DERConstructedSequence)data.getObjectAt(1));
    }

    public DERConstructedSet getRecipientInfo()
	{
			return recipient;
	}

    public DERObject getContent()
    {
        if (data.getSize() == 3)
        {
            DERTaggedObject o = (DERTaggedObject)data.getObjectAt(2);

            return o.getObject();
        }

        return null;
    }

    public BERTaggedObject getContentBER()
    {
        if (data.getSize() == 3)
        {
            BERTaggedObject o = (BERTaggedObject)data.getObjectAt(2);

            return o;
        }

        return null;
    }
    
    
    public DERObject getDERObject()
    {
        DERConstructedSequence  seq = new DERConstructedSequence();

        seq.addObject(new DERInteger(0));
        seq.addObject(recipient);
        seq.addObject(data);

        return seq;
    }
}
