package org.bouncycastle.asn1.pkcs;

import java.util.Enumeration;

import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.x509.*;

/**
 * a PKCS#7 signed data object.
 */
public class SignedData
    implements DEREncodable, PKCSObjectIdentifiers
{
    private DERInteger              version;
    private DERObject               digestAlgorithms;
    private ContentInfo             contentInfo;
    private DERConstructedSequence      certificates;  // SET?
    private DERConstructedSequence      crls;           // SET?
    private DERConstructedSet       signerInfos;

    public SignedData(
        DERConstructedSequence  obj)
    {
        Enumeration     e = obj.getObjects();

        version = (DERInteger)e.nextElement();

        //check whether there is a digestAlgorithm
        DERSet digestAlgorithmsSet = ((DERSet)e.nextElement());
        if ( digestAlgorithmsSet.getSize() > 0) {
            digestAlgorithms = digestAlgorithmsSet.getSequence();
        }
        
        contentInfo = new ContentInfo((DERConstructedSequence)e.nextElement());

        while (e.hasMoreElements())
        {
            DERObject o = (DERObject)e.nextElement();

            //
            // this tries to guess what the following objects 
            //
            if (o instanceof DERTaggedObject)
            {
                DERTaggedObject tagged = (DERTaggedObject)o;

                switch (tagged.getTagNo())
                {
                case 0:
                    //fix: DERConstrcutedSet -> DERConstructedSequence
                    certificates = (DERConstructedSequence)tagged.getObject();
                    break;
                case 1:
                    //fix: DERConstructedSet -> DERConstructedSequence
                    crls = (DERConstructedSequence)tagged.getObject();
                    break;
                default:
                    throw new IllegalArgumentException("unknown tag value " + tagged.getTagNo());
                }
            } else {
				signerInfos = (DERConstructedSet)o;
	    	}
        }
    }

    public SignedData( DERConstructedSet algorithms, ContentInfo contentInfo, 
					   DERConstructedSequence certificates, DERConstructedSequence crls,
					   DERConstructedSet signerInfos) {

			this.digestAlgorithms = algorithms; 
			this.contentInfo = contentInfo;
			this.certificates = certificates;
			this.crls = crls;
			this.signerInfos = signerInfos;
			
	}

    public DERInteger getVersion()
    {
        return version;
    }

    public ContentInfo getContentInfo()
    {
        return contentInfo;
    }

    public DERConstructedSequence getCertficates()
    {
        return certificates;
    }

    public DERConstructedSet getSignerInfos()
    {
        return signerInfos;
    }

    /**
     * <pre>
     * Put the ASN.1 Type from PKCS#7 here
     * </pre>
     */
    public DERObject getDERObject()
    {
        DERConstructedSequence  seq = new DERConstructedSequence();

        seq.addObject( new DERInteger( 1 ) );
        seq.addObject(digestAlgorithms);
        seq.addObject(contentInfo);

        if (certificates != null)
        {
            seq.addObject(new DERTaggedObject(false, 0, certificates));
        }

        if (crls != null)
        {
            seq.addObject(new DERTaggedObject(false, 1, crls));
        }

		seq.addObject(signerInfos);

        return seq;
    }
}
