package org.bouncycastle.crypto.test;

import org.bouncycastle.util.test.*;

import org.bouncycastle.crypto.*;
import org.bouncycastle.crypto.digests.*;
import org.bouncycastle.util.encoders.Hex;

/**
 * Tiger Digest Test
 */
public class TigerDigestTest
    implements Test
{
    final static String[] messages = {
        "",
        "abc",
        "Tiger",
        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-",
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ=abcdefghijklmnopqrstuvwxyz+0123456789",
        "Tiger - A Fast New Hash Function, by Ross Anderson and Eli Biham, proceedings of Fast Software Encryption 3, Cambridge, 1996.",
        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-"
    };

    final static String[] digests = {
        "24F0130C63AC933216166E76B1BB925FF373DE2D49584E7A",
        "F258C1E88414AB2A527AB541FFC5B8BF935F7B951C132951",
        "9F00F599072300DD276ABB38C8EB6DEC37790C116F9D2BDF",
        "87FB2A9083851CF7470D2CF810E6DF9EB586445034A5A386",
        "467DB80863EBCE488DF1CD1261655DE957896565975F9197",
        "3D9AEB03D1BD1A6357B2774DFD6D5B24DD68151D503974FC",
        "00B83EB4E53440C576AC6AAEE0A7485825FD15E70A59FFE4"
    };

    final static String hash64k = "8EF43951B3F5F4FD1D41AFE51B420E710462F233C3AAA8E1";

    public static boolean arraysEqual(byte[] a, byte[] b)
    {
        if (a == b) return true;
        if (a.length != b.length) return false;

        for (int i = 0; i < a.length; i++)
        {
            if (a[i] != b[i]) return false;
        }

        return true;
    }
        
    public String getName()
    {
        return "Tiger";
    }

    public TestResult perform()
    {
        Digest digest = new TigerDigest();
        byte[] resBuf = new byte[digest.getDigestSize()];
        int failCount = 0;

        for (int i = 0; i < messages.length; i++)
        {
            byte[] m = messages[i].getBytes();
            digest.update(m, 0, m.length);
            digest.doFinal(resBuf, 0);

            if (!arraysEqual(resBuf, Hex.decode(digests[i])))
            {
                return new SimpleTestResult(false, getName() + ": Vector " + i + " failed");
            }
        }

        //
        // test 2
        //
		byte[] m = messages[messages.length-1].getBytes();

        digest.update(m, 0, m.length/2);

		// clone the Digest
		Digest d = new TigerDigest((TigerDigest)digest);

        digest.update(m, m.length/2, m.length - m.length/2);
        digest.doFinal(resBuf, 0);

		if (!arraysEqual(resBuf, Hex.decode(digests[digests.length-1])))
        {
            return new SimpleTestResult(false,
                "Tiger failing clone test"
                + System.getProperty("line.separator")
                + "    expected: " + digests[digests.length-1]
                + System.getProperty("line.separator")
                + "    got     : " + new String(Hex.encode(resBuf)));
        }

        d.update(m, m.length/2, m.length - m.length/2);
        d.doFinal(resBuf, 0);

		if (!arraysEqual(resBuf, Hex.decode(digests[digests.length-1])))
        {
            return new SimpleTestResult(false,
                "Tiger failing clone test - part 2"
                + System.getProperty("line.separator")
                + "    expected: " +  digests[digests.length-1]
                + System.getProperty("line.separator")
                + "    got     : " + new String(Hex.encode(resBuf)));
        }

        for (int i = 0; i < 65536; i++)
        {
            digest.update((byte)(i & 0xff));
        }
        digest.doFinal(resBuf, 0);

        if (!arraysEqual(resBuf, Hex.decode(hash64k)))
        {
            return new SimpleTestResult(false, getName() + ": Million a's failed");
        }

        return new SimpleTestResult(true, getName() + ": Okay");
    }

    public static void main(
        String[]    args)
    {
        TigerDigestTest test = new TigerDigestTest();
        TestResult          result = test.perform();

        System.out.println(result);
    }
}
